/*
 Copyright AK

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

 */

package org.jpn.xucker.html;
import java.io.File;
import java.net.URL;

 /**
 * @author AK (www.xucker.jpn.org)
 *
R[h̑org.apache.commons.io.FilenameUtilsɂĂ܂B
  * FilenameUtils[Xł͂ȂAR[h悭ς̂ŁAQƂ͂Ă܂B
  
  2004.08.16 v0.2
  c:\ ̂悤ȃpXɏΉ
 */

public class URLPathNameUtil {
    private URLPathNameUtil(){
    
    }

public static boolean isRemotePath(String path){
    //# or ?
   int sharp=path.indexOf("#");
   int question=path.indexOf("?");
   int colon=path.indexOf(":");
   return colon!=-1 || (colon>sharp || colon>question);
   
}
public static boolean isAbsolutePath(String path){
    return path.startsWith("/");
}
public static File parseURLPathFile(File rootDir,File currentFile,String currentPath,String path){
    if(path==null){
        //if path ==null  err.
        throw new NullPointerException();
    }else if(isRemotePath(path)){
        //if path is remotePath,this return null.
        return null;
    }
    
    File result=null;
    String purePath=getNonQueryAnchorPath(path);
    
    if(rootDir==null && isAbsolutePath(purePath)){
       return null;
    }
    
    if(rootDir!=null && isAbsolutePath(purePath)){
        //parse Absolute Path
        result=new File(rootDir,purePath.substring(1));
    }
    
    if(currentFile!=null && !isAbsolutePath(purePath)){
        //parse local path;
      
        if(purePath.equals("")){
            return currentFile;
        }
       
        if(currentPath==null || rootDir==null){
            String currentFilePath=createPath(currentFile.getAbsolutePath(),purePath);
            if(currentFilePath!=null){
                result=new File(currentFilePath);
            }
           
        }else{
        String newPath=createPath(currentPath,purePath);
        if(newPath!=null){
            result=new File(rootDir,newPath);
        	}
        }
        
    }
    return result;
}
	/**
	 * t@C̊gqo܂B
	 * @param filename
	 * @return t@C̊gq
	 */
	/*
	 * 
	 * Thieved from org.apache.commons.io.FilenameUtils
	 */
	 
	public static String getExtension(String filename) {
		int index = filename.lastIndexOf('.');

		if (-1 == index) {
			return "";
		} else {
			return filename.substring(index + 1);
		}
	}
	/**t@C̊gq̂擾܂B
	 *  
	 * @param filename
	 * @return gq
	 */
	/*
		 * 
		 * Thieved from org.apache.commons.io.FilenameUtils
		 */
	public static String removeExtension(String filename) {
		int index = filename.lastIndexOf('.');

		if (-1 == index) {
			return filename;
		} else {
			return filename.substring(0, index);
		}
	}
	/**
	 * t@C̖Oo܂B
	 * @param filepath
	 * @return t@C̖Ô݂Ԃ
	 */
	/*
		 * 
		 * Thieved from org.apache.commons.io.FilenameUtils
		 */
	public static String getFileName(String filepath) {
		return getFileName(filepath, File.separatorChar);
	}
	/**t@C̖Oo܂B
	 * @param filepath
	 * @param fileSeparatorChar
	 * @return t@C̖Ô݂Ԃ
	 */
	/*
		 * 
		 * Thieved from org.apache.commons.io.FilenameUtils
		 */
	public static String getFileName(String filepath, char fileSeparatorChar) {
		int index = filepath.lastIndexOf(fileSeparatorChar);

		if (-1 == index) {
			return filepath;
		} else {
			return filepath.substring(index + 1);
		}
	}
	/**
	 * t@C̃fBNg[o܂B
	 * @param filepath
	 * @return t@C̃fBNg[Ԃ܂B
	 */
	/*
		 * 
		 * Thieved from org.apache.commons.io.FilenameUtils
		 */
	public static String getParentPath(String filepath) {
		return getParentPath(filepath, File.separatorChar);
	}
	/**
	 * t@C̃fBNg[o܂B
	 * @param filepath
	 * @param fileSeparatorChar
	 * @return t@C̃fBNg[Ԃ܂B
	 */
	/*
		 * 
		 * Thieved from org.apache.commons.io.FilenameUtils
		 */
	public static String getParentPath(
		String filepath,
		char fileSeparatorChar) {
		int index = filepath.lastIndexOf(fileSeparatorChar);

		if (-1 == index) {
			return filepath;
		} else {
			return filepath.substring(index + 1);
		}
	}

	/**
	 * path1x[Xɂpath2킹pXo܂B
	 * @param path1
	 * @param path2
	 * @return path1x[Xɂpath2킹pX
	 * 
	 * URLł̃pX̌C[WĂ܂B
	 * "/" @̊Kw͂܂B
	 */
	public static String createPath(String path1, String path2) {
		String basePath = fixBasePath(path1);
		String resultPath = null;
		if (path1.indexOf("/") != -1)
			resultPath = catPath(basePath, path2);
		else {
		    if(path1.indexOf("\\")!=-1){
		    String tmpPath1=path1.replaceAll("\\\\","/");
		    try{
		        resultPath=catPath(tmpPath1,path2);
		    }catch(Exception e){
		        return null;
		    }
		    if(resultPath==null){
		        return null;
		    }
		    resultPath=resultPath.replaceAll("/","\\\\");
		    }else{
			resultPath = path2;
		    }
			
		}

		if (resultPath == null) {
			String fileName = getFileName(path2, '/');
			if (fileName != null)
				resultPath = "/" + fileName;
			else
				resultPath = "/";
		}else{
			resultPath=fixBasePath(resultPath);
		}
		
		return normalize(resultPath);
	}

	/**
	 * path1x[Xɂpath2킹pXo܂B
	 * @param basePath
	 * @param newPath
	 * @return path1x[Xɂpath2킹pX
	 * 
	 * URLł̃pX̌C[WĂ܂B
	 * "/" @̊Kw͂܂B
	 */
	public static String getRemainPath(String basePath,String newPath){
		String ret=newPath;
		if(newPath.startsWith(basePath)){
			ret=newPath.substring(basePath.length());
			if(ret.startsWith("\\") || ret.startsWith("/"))
			ret=ret.substring(1);
		}
		return ret;
	}
	
	private static String fixBasePath(String path) {
		String result = path;
		while (true) {
			int skip = 0;
			if (result.startsWith("../")) {
				skip = 3;
			} else if (result.startsWith("./")) {
				skip = 2;
			}
			if (skip == 0)
				break;
			else
				result = result.substring(skip);
		}
		if (path.equals(result))
			return path;
		else
			return "/" + result;
	}

	/*
		 * 
		 * Thieved from org.apache.commons.io.FilenameUtils
		 */
	private static String catPath(String lookupPath, String path) {
		String result=null;
		try{
		// Cut off the last slash and everything beyond
		int index = lookupPath.lastIndexOf("/");
		String lookup = lookupPath.substring(0, index);
		String pth = path;

		// Deal with .. by chopping dirs off the lookup path
		while (pth.startsWith("../")) {
			if (lookup.length() > 0 ) {
				index = lookup.lastIndexOf("/");
				if(index==-1)
				    return null;
				lookup = lookup.substring(0, index);
			} else {
				// More ..'s than dirs, return null
				return null;
			}

			index = pth.indexOf("../") + 3;
			pth = pth.substring(index);
		}

		result=new StringBuffer(lookup).append("/").append(pth).toString();
		}catch(Exception e){
			e.printStackTrace();
		}
		return result;
	}

	/*
	 * 
	 * Thieved from org.apache.commons.io.FilenameUtils
	 */
	private static String normalize(String path) {
		String normalized = path;
		// Resolve occurrences of "//" in the normalized path
		while (true) {
			int index = normalized.indexOf("//");
			if (index < 0)
				break;
			normalized =
				normalized.substring(0, index)
					+ normalized.substring(index + 1);
		}

		// Resolve occurrences of "/./" in the normalized path
		while (true) {
			int index = normalized.indexOf("/./");
			if (index < 0)
				break;
			normalized =
				normalized.substring(0, index)
					+ normalized.substring(index + 2);
		}

		// Resolve occurrences of "/../" in the normalized path
		while (true) {
			int index = normalized.indexOf("/../");
			if (index < 0)
				break;
			if (index == 0)
				return null; // Trying to go outside our context
			int index2 = normalized.lastIndexOf('/', index - 1);
			normalized =
				normalized.substring(0, index2)
					+ normalized.substring(index + 3);
		}

		// Return the normalized path that we have completed
		return normalized;
	}
	
	public static File searchFileAroundClass(Class classObject,String fileName){
			ClassLoader loader=classObject.getClassLoader();
			String path=classObject.getName().replaceAll("\\.","/");
						path=path+".class";
		
			URL url=loader.getResource(path);
			String classPath=url.toString();
			System.out.println(classPath);
			if(classPath.startsWith("jar:")){
				int last=classPath.lastIndexOf("!");
				classPath=classPath.substring("jar:".length(),last);
			}
			System.out.println(classPath);
			if(classPath.startsWith("file:/")){
				File checkFile=new File(classPath.substring("file:/".length())).getParentFile();
				while(true){
					File tmp=new File(checkFile,fileName);
					System.out.println(tmp.getAbsolutePath());
					if(tmp.exists())
						return tmp;
					else{
						checkFile=checkFile.getParentFile();
						if(checkFile==null)
						break;
					}
				}
				return null;
			}else{
				return null;
			}
		}
	public static String getNonQueryAnchorPath(String path){
		String anchor=getAnchor(path);
		if(anchor==null){
			String query=getQuery(path);
			if(query==null){
				return path;
			}else{
				return path.substring(0,path.length()-query.length()-1);
			}
		}else{
			String query=getQuery(path);
						if(query==null){
							return path.substring(0,path.length()-anchor.length()-1);
						}else{
							return path.substring(0,path.length()-query.length()-anchor.length()-2);
						}
			
		}
	}
	public static String getQuery(String path){
		String anchor=getAnchor(path);
		
		if(anchor==null){
			int index=path.lastIndexOf("?");
			if(index!=-1){
				return path.substring(index+1);
			}
		}else{
			int index=path.lastIndexOf("?",path.length()-anchor.length());
			if(index!=-1){
			return path.substring(index+1,path.length()-anchor.length()-1);
			}
		}
		
		return null;
	}
	
	public static String getAnchor(String path){
		int index=path.lastIndexOf("#");
				if(index!=-1){
					//anchor last
					return path.substring(index+1);
				}
				return null;
	}
}
